"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var os = require("os");
var capitalize = require("capitalize");
var prettier = require("prettier");
function getTypeFromGraphQLType(type) {
    if (type === "Int" || type === "Float") {
        return "number";
    }
    if (type === "Boolean") {
        return "boolean";
    }
    if (type === "String" || type === "ID" || type === "DateTime") {
        return "string";
    }
    return "string";
}
function format(code, options) {
    if (options === void 0) { options = {}; }
    try {
        return prettier.format(code, __assign({}, options, { parser: "typescript" }));
    }
    catch (e) {
        console.log("There is a syntax error in generated code, unformatted code printed, error: " + JSON.stringify(e));
        return code;
    }
}
exports.format = format;
function printFieldLikeType(field, lookupType) {
    if (lookupType === void 0) { lookupType = true; }
    if (field.type.isScalar) {
        return "" + getTypeFromGraphQLType(field.type.name) + (field.type.isArray ? "[]" : "") + (!field.type.isRequired ? "| null" : "");
    }
    return lookupType
        ? "T['" + field.type.name + (field.type.isEnum || field.type.isUnion ? "" : "Root") + "']" + (field.type.isArray ? "[]" : "") + (!field.type.isRequired ? "| null" : "")
        : "" + field.type.name + (field.type.isEnum || field.type.isUnion ? "" : "Root") + (field.type.isArray ? "[]" : "") + (!field.type.isRequired ? "| null" : "");
}
exports.printFieldLikeType = printFieldLikeType;
function generate(args) {
    return "\nimport { GraphQLResolveInfo } from 'graphql'\n\nexport interface ResolverFn<Root, Args, Ctx, Payload> {\n  (root: Root, args: Args, ctx: Ctx, info: GraphQLResolveInfo):\n    | Payload\n    | Promise<Payload>\n}\n\nexport interface ITypes {\nContext: any\n" + args.enums.map(function (e) { return e.name + ": any"; }).join(os.EOL) + "\n" + args.unions.map(function (union) { return union.name + ": any"; }).join(os.EOL) + "\n" + args.types.map(function (type) { return type.name + "Root: any"; }).join(os.EOL) + "\n}\n\n  " + args.types
        .map(function (type) { return "export namespace I" + type.name + " {\n  " + type.fields
        .map(function (field) { return (field.arguments.length > 0
        ? "export interface Args" + capitalize(field.name) + " {\n      " + field.arguments
            .map(function (arg) { return arg.name + ": " + printFieldLikeType(arg); })
            .join(os.EOL) + "\n    }"
        : "") + "\n\n  export type " + capitalize(field.name) + "Resolver<T extends ITypes> = ResolverFn<\n    T['" + type.name + (type.type.isEnum || type.type.isUnion ? "" : "Root") + "'],\n    " + (field.arguments.length > 0 ? "Args" + capitalize(field.name) : "{}") + ",\n    T['Context'],\n    " + printFieldLikeType(field) + "\n  >\n  "; })
        .join(os.EOL) + "\n\n  export interface Resolver<T extends ITypes> {\n  " + type.fields
        .map(function (field) { return field.name + ": " + capitalize(field.name) + "Resolver<T>"; })
        .join(os.EOL) + "\n  }\n}\n"; })
        .join(os.EOL) + "\n\nexport interface IResolvers<T extends ITypes> {\n  " + args.types
        .map(function (type) { return type.name + ": I" + type.name + ".Resolver<T>"; })
        .join(os.EOL) + "\n}\n\n  ";
}
exports.generate = generate;
//# sourceMappingURL=ts-generator.js.map